@@ -1,17 +1,41 @@
 //! Functions for interacting with the execution data passed to PHP functions\
 //! introduced in Rust.
 
+use std::mem;
+
 use crate::{
     bindings::{zend_execute_data, ZEND_MM_ALIGNMENT, ZEND_MM_ALIGNMENT_MASK},
     errors::{Error, Result},
 };
 
-use super::types::{object::ZendObject, zval::Zval};
+use super::types::{
+    object::{ClassObject, RegisteredClass, ZendClassObject, ZendObject},
+    zval::Zval,
+};
 
 /// Execution data passed when a function is called from Zend.
 pub type ExecutionData = zend_execute_data;
 
 impl ExecutionData {
+    /// Attempts to retrieve a reference to the underlying class object of the Zend object.
+    ///
+    /// Returns a [`ClassObject`] if the execution data contained a valid object, otherwise
+    /// returns [`None`].
+    ///
+    /// # Safety
+    ///
+    /// The caller must guarantee that the function is called on an instance of [`ExecutionData`]
+    /// that:
+    ///
+    /// 1. Contains an object.
+    /// 2. The object was originally derived from `T`.
+    pub unsafe fn get_object<T: RegisteredClass>(&self) -> Option<ClassObject<'static, T>> {
+        let ptr = self.This.object()? as *const ZendObject as *mut u8;
+        let offset = mem::size_of::<T>();
+        let ptr = ptr.offset(0 - offset as isize) as *mut ZendClassObject<T>;
+        Some(ClassObject::from_zend_class_object(&mut *ptr, false))
+    }
+
     /// Attempts to retrieve the 'this' object, which can be used in class methods
     /// to retrieve the underlying Zend object.
     pub fn get_self(&self) -> Result<&mut ZendObject> {
