@@ -4,6 +4,9 @@
 pub mod array;
 pub mod binary;
 pub mod callable;
+#[cfg(any(docs, feature = "closure"))]
+#[cfg_attr(docs, doc(cfg(feature = "closure")))]
+pub mod closure;
 pub mod long;
 pub mod object;
 pub mod string;
