@@ -29,6 +29,9 @@ use super::{array::ZendHashTable, callable::Callable, object::ZendObject};
 /// Zend value. Represents most data types that are in the Zend engine.
 pub type Zval = zval;
 
+unsafe impl Send for Zval {}
+unsafe impl Sync for Zval {}
+
 impl<'a> Zval {
     /// Creates a new, empty zval.
     pub(crate) const fn new() -> Self {
@@ -183,7 +186,7 @@ impl<'a> Zval {
     /// # Parameters
     ///
     /// * `params` - A list of parameters to call the function with.
-    pub fn try_call(&self, params: Vec<&dyn IntoZval>) -> Result<Zval> {
+    pub fn try_call(&self, params: Vec<&dyn IntoZvalDyn>) -> Result<Zval> {
         self.callable().ok_or(Error::Callable)?.try_call(params)
     }
 
@@ -266,13 +269,13 @@ impl<'a> Zval {
         Ok(())
     }
 
-    /// Sets the value of the zval as a binary string.
+    /// Sets the value of the zval as a binary string, which is represented in Rust as a vector.
     ///
     /// # Parameters
     ///
     /// * `val` - The value to set the zval as.
-    pub fn set_binary<T: Pack, U: AsRef<[T]>>(&mut self, val: U) {
-        let ptr = T::pack_into(val.as_ref().to_vec());
+    pub fn set_binary<T: Pack>(&mut self, val: Vec<T>) {
+        let ptr = T::pack_into(val);
         self.value.str_ = ptr;
         self.u1.type_info = ZvalTypeFlags::StringEx.bits();
     }
@@ -339,13 +342,13 @@ impl<'a> Zval {
         self.value.res = val;
     }
 
-    /// Sets the value of the zval as an object.
+    /// Sets the value of the zval as a reference to an object.
     ///
     /// # Parameters
     ///
     /// * `val` - The value to set the zval as.
-    /// * `copy` - Whether to copy the object or pass as a reference.
-    pub fn set_object(&mut self, val: &ZendObject, _copy: bool) {
+    pub fn set_object(&mut self, val: &mut ZendObject) {
+        val.refcount_inc();
         self.u1.type_info = ZvalTypeFlags::ObjectEx.bits();
         self.value.obj = (val as *const ZendObject) as *mut ZendObject;
     }
@@ -360,11 +363,6 @@ impl<'a> Zval {
         self.value.arr = val.into_ptr();
     }
 
-    /// Sets the reference count of the Zval.
-    pub(crate) fn set_refcount(&mut self, rc: u32) {
-        unsafe { (*self.value.counted).gc.refcount = rc }
-    }
-
     /// Used to drop the Zval but keep the value of the zval intact.
     ///
     /// This is important when copying the value of the zval, as the actual value
@@ -395,7 +393,7 @@ impl Debug for Zval {
                 DataType::True => field!(true),
                 DataType::Long => field!(self.long()),
                 DataType::Double => field!(self.double()),
-                DataType::String => field!(self.string()),
+                DataType::String | DataType::Mixed => field!(self.string()),
                 DataType::Array => field!(self.array()),
                 DataType::Object => field!(self.object()),
                 DataType::Resource => field!(self.resource()),
@@ -422,14 +420,17 @@ impl Drop for Zval {
 /// Provides implementations for converting Rust primitive types into PHP zvals. Alternative to the
 /// built-in Rust [`From`] and [`TryFrom`] implementations, allowing the caller to specify whether
 /// the Zval contents will persist between requests.
-pub trait IntoZval {
+pub trait IntoZval: Sized {
+    /// The corresponding type of the implemented value in PHP.
+    const TYPE: DataType;
+
     /// Converts a Rust primitive type into a Zval. Returns a result containing the Zval if
     /// successful.
     ///
     /// # Parameters
     ///
     /// * `persistent` - Whether the contents of the Zval will persist between requests.
-    fn as_zval(&self, persistent: bool) -> Result<Zval> {
+    fn into_zval(self, persistent: bool) -> Result<Zval> {
         let mut zval = Zval::new();
         self.set_zval(&mut zval, persistent)?;
         Ok(zval)
@@ -442,11 +443,38 @@ pub trait IntoZval {
     ///
     /// * `zv` - The Zval to set the content of.
     /// * `persistent` - Whether the contents of the Zval will persist between requests.
-    fn set_zval(&self, zv: &mut Zval, persistent: bool) -> Result<()>;
+    fn set_zval(self, zv: &mut Zval, persistent: bool) -> Result<()>;
+}
+
+/// An object-safe version of the [`IntoZval`] trait.
+///
+/// This trait is automatically implemented on any type that implements both [`IntoZval`] and [`Clone`].
+/// You avoid implementing this trait directly, rather implement these two other traits.
+pub trait IntoZvalDyn {
+    /// Converts a Rust primitive type into a Zval. Returns a result containing the Zval if
+    /// successful. `self` is cloned before being converted into a zval.
+    ///
+    /// # Parameters
+    ///
+    /// * `persistent` - Whether the contents of the Zval will persist between requests.
+    fn as_zval(&self, persistent: bool) -> Result<Zval>;
+
+    /// Returns the PHP type of the type.
+    fn get_type(&self) -> DataType;
+}
+
+impl<T: IntoZval + Clone> IntoZvalDyn for T {
+    fn as_zval(&self, persistent: bool) -> Result<Zval> {
+        self.clone().into_zval(persistent)
+    }
+
+    fn get_type(&self) -> DataType {
+        Self::TYPE
+    }
 }
 
 macro_rules! into_zval {
-    ($type: ty, $fn: ident) => {
+    ($type: ty, $fn: ident, $dt: ident) => {
         impl From<$type> for Zval {
             fn from(val: $type) -> Self {
                 let mut zv = Self::new();
@@ -456,25 +484,27 @@ macro_rules! into_zval {
         }
 
         impl IntoZval for $type {
-            fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
-                zv.$fn(*self);
+            const TYPE: DataType = DataType::$dt;
+
+            fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
+                zv.$fn(self);
                 Ok(())
             }
         }
     };
 }
 
-into_zval!(i8, set_long);
-into_zval!(i16, set_long);
-into_zval!(i32, set_long);
+into_zval!(i8, set_long, Long);
+into_zval!(i16, set_long, Long);
+into_zval!(i32, set_long, Long);
 
-into_zval!(u8, set_long);
-into_zval!(u16, set_long);
+into_zval!(u8, set_long, Long);
+into_zval!(u16, set_long, Long);
 
-into_zval!(f32, set_double);
-into_zval!(f64, set_double);
+into_zval!(f32, set_double, Double);
+into_zval!(f64, set_double, Double);
 
-into_zval!(bool, set_bool);
+into_zval!(bool, set_bool, Bool);
 
 macro_rules! try_into_zval_int {
     ($type: ty) => {
@@ -490,8 +520,10 @@ macro_rules! try_into_zval_int {
         }
 
         impl IntoZval for $type {
-            fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
-                let val: ZendLong = (*self).try_into().map_err(|_| Error::IntegerOverflow)?;
+            const TYPE: DataType = DataType::Long;
+
+            fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
+                let val: ZendLong = self.try_into().map_err(|_| Error::IntegerOverflow)?;
                 zv.set_long(val);
                 Ok(())
             }
@@ -519,8 +551,10 @@ macro_rules! try_into_zval_str {
         }
 
         impl IntoZval for $type {
-            fn set_zval(&self, zv: &mut Zval, persistent: bool) -> Result<()> {
-                zv.set_string(self, persistent)
+            const TYPE: DataType = DataType::String;
+
+            fn set_zval(self, zv: &mut Zval, persistent: bool) -> Result<()> {
+                zv.set_string(&self, persistent)
             }
         }
     };
@@ -529,11 +563,22 @@ macro_rules! try_into_zval_str {
 try_into_zval_str!(String);
 try_into_zval_str!(&str);
 
+impl IntoZval for () {
+    const TYPE: DataType = DataType::Null;
+
+    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
+        zv.set_null();
+        Ok(())
+    }
+}
+
 impl<T> IntoZval for Option<T>
 where
     T: IntoZval,
 {
-    fn set_zval(&self, zv: &mut Zval, persistent: bool) -> Result<()> {
+    const TYPE: DataType = T::TYPE;
+
+    fn set_zval(self, zv: &mut Zval, persistent: bool) -> Result<()> {
         match self {
             Some(val) => val.set_zval(zv, persistent),
             None => {
@@ -545,8 +590,10 @@ where
 }
 
 impl<'a> IntoZval for ZendHashTable<'a> {
-    fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
-        zv.set_array(self.clone());
+    const TYPE: DataType = DataType::Array;
+
+    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
+        zv.set_array(self);
         Ok(())
     }
 }
@@ -555,7 +602,9 @@ impl<T> IntoZval for Vec<T>
 where
     T: IntoZval,
 {
-    fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
+    const TYPE: DataType = DataType::Array;
+
+    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
         let hm = self
             .try_into()
             .map_err(|_| Error::ZvalConversion(DataType::Array))?;
@@ -569,7 +618,9 @@ where
     K: AsRef<str>,
     V: IntoZval,
 {
-    fn set_zval(&self, zv: &mut Zval, _: bool) -> Result<()> {
+    const TYPE: DataType = DataType::Array;
+
+    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
         let hm = self.try_into()?;
         zv.set_array(hm);
         Ok(())
